//*****************************************************************************
// Copyright 2017-2019 Intel Corporation
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//*****************************************************************************

#include "ngraph/op/cum_sum.hpp"
#include "ngraph/graph_util.hpp"
#include "ngraph/op/broadcast.hpp"
#include "ngraph/op/constant.hpp"

using namespace std;
using namespace ngraph;

const string op::CumSum::type_name{"CumSum"};

op::CumSum::CumSum(const Output<Node>& arg,
                   const Output<Node>& axis,
                   const bool exclusive,
                   const bool reverse)
    : Op({arg, axis})
    , m_exclusive(exclusive)
    , m_reverse(reverse)
{
    NODE_VALIDATION_CHECK(this,
                          axis.get_element_type() == element::i32 ||
                              axis.get_element_type() == element::i64,
                          "axis element type must be either int64_t or int32_t but got (",
                          axis.get_element_type(),
                          ").");
    set_output_type(0, arg.get_element_type(), arg.get_shape());
}

shared_ptr<Node> op::CumSum::copy_with_new_args(const NodeVector& new_args) const
{
    check_new_args_count(this, new_args);
    return make_shared<op::CumSum>(new_args.at(0), new_args.at(1), m_exclusive, m_reverse);
}

shared_ptr<Node> op::CumSum::get_default_value() const
{
    return ngraph::make_constant_from_string("0", get_element_type(), get_shape());
}
